from django.core import management
from django.test import TestCase


class CommandTestCase(TestCase):
    """
    Test cases for django-ph-geography custom commands
    """

    def run_command_phgeofixtures(self):
        """Run custom manage.py command 'phgeofixtures'"""
        management.call_command('phgeofixtures', verbosity=0)
        return True

    def test_command_phgeofixtures(self):
        self.assertTrue(self.run_command_phgeofixtures())



from django.core.exceptions import FieldDoesNotExist
from django.db import models


class PhilippineGeography(models.Model):
    """
    Abstract model for common geographical fields.

    (Also a Yoyoy Villame song: https://www.youtube.com/watch?v=e_PMcFjE9as)

    Available fields are:
        * code - Unique geographical code.
        * name - Geographical name.
        * population - Population count based on 2015 POPCEN.
                        Null value  means no data is available.
        * is_active - Toggle if entry is active (True) or not (False).
    """
    code = models.CharField(max_length=10, unique=True, null=False, verbose_name='Code')
    name = models.CharField(max_length=100, null=False, verbose_name='Name')
    population = models.PositiveIntegerField(null=True, verbose_name='Population')
    is_active = models.BooleanField(null=False, default=True, verbose_name='Is Active')

    class Meta:
        abstract = True

    @classmethod
    def add_field(cls, name, value):
        """
        Add a field to the model.

        Using this method to the abstract model will apply the action to all subclasses.
        """
        if cls._meta.abstract:
            for subcls in cls.__subclasses__():
                subcls.add_to_class(name, value)
        else:
            cls.add_to_class(name, value)

    @classmethod
    def _remove_field(cls, names):
        """
        Remove fields from the model.
        """
        for name in names:
            try:
                field = cls._meta.get_field(name)
            except FieldDoesNotExist:
                raise AttributeError('No attribute name found in model: {name}'.format(name=name))
            else:
                cls._meta.local_fields.remove(field)

    @classmethod
    def remove_field(cls, *names):
        """
        Remove fields from the model.

        Supports *args of <str> field names.

        Using this method to the abstract model will apply the action to all subclasses.
        """

        if not all(isinstance(name, str) for name in names):
            raise TypeError('"remove_field" only supports <str> of field names.')

        if cls._meta.abstract:
            is_removed = False
            for subcls in cls.__subclasses__():
                try:
                    subcls._remove_field(names)
                except AttributeError:
                    pass
                else:
                    is_removed = True
            if not is_removed:
                raise AttributeError('No attribute name found in any model: {names}'.format(names=', '.join(names)))
        else:
            cls._remove_field(names)

    def __repr__(self):
        code = getattr(self, 'code', None)
        name = getattr(self, 'name', None)
        model = self.__class__.__name__
        if code and name:
            value = '<Code: {code}, {model}: {name}>'.format(code=code, model=model, name=name)
        elif name:
            value = '<{model}: {name}>'.format(model=model, name=name)
        else:
            value = super(PhilippineGeography, self).__repr__()
        return value

    def __str__(self):
        return getattr(self, 'name', None) or super(PhilippineGeography, self).__str__()


class Region(PhilippineGeography):
    """
    Model for regions.

    Available fields are:
        * code - Unique geographical code for region.
        * name - Geographical name for region.
        * population - Population count based on 2015 POPCEN.
                        Null value means no data is available.
        * island_group - Island group where the region is located. Possible values are 'L' (for Luzon),
                         'V' (for Visayas), and 'M' (for Mindanao).
        * is_active - Toggle if region is active (True) or not (False).
    """
    ISLAND_GROUP_LUZON = 'L'
    ISLAND_GROUP_VISAYAS = 'V'
    ISLAND_GROUP_MINDANAO = 'M'
    ISLAND_GROUP_CHOICES = (
        (ISLAND_GROUP_LUZON, 'LUZON'),
        (ISLAND_GROUP_VISAYAS, 'VISAYAS'),
        (ISLAND_GROUP_MINDANAO, 'MINDANAO'),
    )

    island_group = models.CharField(max_length=1, choices=ISLAND_GROUP_CHOICES,
                                    null=False, verbose_name='Island Group')

    class Meta:
        db_table = 'ph_geography_region'
        verbose_name = 'Region'
        verbose_name_plural = 'Regions'


class Province(PhilippineGeography):
    """
    Model for provinces.

    Available fields are:
        * code - Unique geographical code for province.
        * name - Geographical name for province.
        * population - Population count based on 2015 POPCEN.
                        Null value means no data is available.
        * region - Region where the province is located.
        * income_class - Income classification. Possible values are '1' (1st), '2' (2nd), '3' (3rd),
                          '4' (4th), '5' (5th), '6' (6th), and 'S' (Special).
                          Blank means no data is available.
        * is_active - Toggle if province is active (True) or not (False).
    """
    INCOME_CLASS_1 = '1'
    INCOME_CLASS_2 = '2'
    INCOME_CLASS_3 = '3'
    INCOME_CLASS_4 = '4'
    INCOME_CLASS_5 = '5'
    INCOME_CLASS_6 = '6'
    INCOME_CLASS_SPECIAL = 'S'
    INCOME_CLASS_CHOICES = (
        (INCOME_CLASS_1, '1ST'),
        (INCOME_CLASS_2, '2ND'),
        (INCOME_CLASS_3, '3RD'),
        (INCOME_CLASS_4, '4TH'),
        (INCOME_CLASS_5, '5TH'),
        (INCOME_CLASS_6, '6TH'),
        (INCOME_CLASS_SPECIAL, 'SPECIAL'),
    )

    region = models.ForeignKey(
        Region,
        related_name='provinces',
        related_query_name='province',
        null=False,
        on_delete=models.CASCADE,
        verbose_name='Region'
    )
    income_class = models.CharField(max_length=1, choices=INCOME_CLASS_CHOICES,
                                    blank=True, null=False, verbose_name='Income Class')

    class Meta:
        db_table = 'ph_geography_province'
        verbose_name = 'Province'
        verbose_name_plural = 'Provinces'

    @property
    def island_group(self):
        return self.region.island_group


class Municipality(PhilippineGeography):
    """
    Model for municipalities and cities.

    Available fields are:
        * code - Unique geographical code for municipality.
        * name - Geographical name for municipality.
        * population - Population count based on 2015 POPCEN.
                        Null value means no data is available.
        * province - Province where the municipality is located.
        * is_city - Toggle to define whether the municipality is a city (True) or not (False).
        * is_capital - Toggle to define whether the city is a capital (True) or not (False).
        * city_class - City legal classification. Possible values are 'C' (component city),
                        'I' (independent component city), and 'H' (highly urbanized city).
                        Blank value means no data is available.
        * income_class - Income classification. Possible values are '1' (1st), '2' (2nd), '3' (3rd),
                          '4' (4th), '5' (5th), '6' (6th), and 'S' (Special).
                          Blank value means no data is available.
        * is_active - Toggle if municipality is active (True) or not (False).
    """
    CITY_CLASS_COMPONENT_CITY = 'C'
    CITY_CLASS_INDEPENDENT_COMPONENT_CITY = 'I'
    CITY_CLASS_HIGHLY_URBANIZED_CITY = 'H'
    CITY_CLASS_CHOICES = (
        (CITY_CLASS_COMPONENT_CITY, 'CC'),
        (CITY_CLASS_INDEPENDENT_COMPONENT_CITY, 'ICC'),
        (CITY_CLASS_HIGHLY_URBANIZED_CITY, 'HUC'),
    )
    INCOME_CLASS_1 = '1'
    INCOME_CLASS_2 = '2'
    INCOME_CLASS_3 = '3'
    INCOME_CLASS_4 = '4'
    INCOME_CLASS_5 = '5'
    INCOME_CLASS_6 = '6'
    INCOME_CLASS_SPECIAL = 'S'
    INCOME_CLASS_CHOICES = (
        (INCOME_CLASS_1, '1ST'),
        (INCOME_CLASS_2, '2ND'),
        (INCOME_CLASS_3, '3RD'),
        (INCOME_CLASS_4, '4TH'),
        (INCOME_CLASS_5, '5TH'),
        (INCOME_CLASS_6, '6TH'),
        (INCOME_CLASS_SPECIAL, 'SPECIAL'),
    )

    province = models.ForeignKey(
        Province,
        related_name='municipalities',
        related_query_name='municipality',
        null=False,
        on_delete=models.CASCADE,
        verbose_name='Province'
    )
    is_city = models.BooleanField(null=False, verbose_name='Is City')
    is_capital = models.BooleanField(null=False, verbose_name='Is Capital')
    city_class = models.CharField(max_length=1, choices=CITY_CLASS_CHOICES,
                                  blank=True, null=False, verbose_name='City Class')
    income_class = models.CharField(max_length=1, choices=INCOME_CLASS_CHOICES,
                                    blank=True, null=False, verbose_name='Income Class')

    class Meta:
        db_table = 'ph_geography_municipality'
        verbose_name = 'Municipality'
        verbose_name_plural = 'Municipalities'

    @property
    def region(self):
        return self.province.region

    @property
    def island_group(self):
        return self.province.region.island_group


class Barangay(PhilippineGeography):
    """
    Model for barangays.

    Available fields are:
        * code - Unique geographical code for barangay.
        * name - Geographical name for barangay.
        * population - Population count based on 2015 POPCEN.
                        Null value means no data is available.
        * municipality - Municipality where the barangay is located.
        * is_urban - Toggle to define whether the barangay is urban (True) or rural (False).
                      Null value means no data is available.
        * is_active - Toggle if barangay is active (True) or not (False).
    """
    municipality = models.ForeignKey(
        Municipality,
        related_name='barangays',
        related_query_name='barangays',
        null=False,
        on_delete=models.CASCADE,
        verbose_name='Barangay'
    )
    is_urban = models.NullBooleanField(null=True, verbose_name='Is Urban')

    class Meta:
        db_table = 'ph_geography_barangay'
        verbose_name = 'Barangay'
        verbose_name_plural = 'Barangays'

    @property
    def province(self):
        return self.municipality.province

    @property
    def region(self):
        return self.province.region

    @property
    def island_group(self):
        return self.region.island_group



import os

from django_migration_testcase import MigrationTest

from django.conf import settings
from django.core import management
from django.core.exceptions import FieldDoesNotExist
from django.db import models

from ph_geography.models import PhilippineGeography
from ph_geography.models import Barangay
from ph_geography.models import Municipality
from ph_geography.models import Province
from ph_geography.models import Region


class MonkeyPatchingTestCase(MigrationTest):
    """
    Test cases for django-ph-geography monkey patching

    Testing these test cases:
        * Adding field to a specific model
        * Adding field to all models (via abstract model)
        * Removing field to a specific model
        * Removing field to all models (via abstract model)
    """
    app_name = 'ph_geography'
    apps_after = None
    before = '0001'
    after = '0002'

    ADDED_FIELD_SPECIFIC = 'specific'
    ADDED_FIELD_ALL = 'all'
    REMOVED_FIELD_SPECIFIC = 'is_urban'
    REMOVED_FIELD_ALL = 'population'
    MIGRATION_FILE_NAME = 'monkey_patching'

    @classmethod
    def setUpClass(cls):
        """Apply monkey patching to the provided models and generate a temporary migration file"""
        super(MonkeyPatchingTestCase, cls).setUpClass()

        # Add field to specific model
        Barangay.add_field(cls.ADDED_FIELD_SPECIFIC, models.BooleanField(default=True))

        # Add field to all models
        PhilippineGeography.add_field(cls.ADDED_FIELD_ALL, models.BooleanField(default=True))

        # Remove field to specific model
        Barangay.remove_field(cls.REMOVED_FIELD_SPECIFIC)

        # Remove field to all models
        PhilippineGeography.remove_field(cls.REMOVED_FIELD_ALL)

        # Remove field 'name' to test __str__ output
        Region.remove_field('name')

        # Remove field 'name' to test __repr__ 1st fallback output
        Province.remove_field('code')

        # Remove field 'name' and 'code' to test __repr__ 2nd fallback output
        Municipality.remove_field('code', 'name')

        management.call_command('makemigrations', 'ph_geography',
                                name=cls.MIGRATION_FILE_NAME, verbosity=0, interactive=False)

    @classmethod
    def tearDownClass(cls):
        """Delete generated migration file after completing testing"""
        path = os.path.join(*[
            settings.BASE_DIR,
            'ph_geography',
            'migrations',
            '{}_{}.py'.format(cls.after, cls.MIGRATION_FILE_NAME),
        ])
        if os.path.isfile(path):
            os.remove(path)
        super(MonkeyPatchingTestCase, cls).tearDownClass()

    def setUp(self):
        """Run migration file and refresh models to reflect the changes"""
        super(MonkeyPatchingTestCase, self).setUp()
        self.run_migration()
        self.barangay_model = self.get_model_after('Barangay')
        self.municipality_model = self.get_model_after('Municipality')
        self.province_model = self.get_model_after('Province')
        self.region_model = self.get_model_after('Region')

    #
    #   __str__ and __repr__
    #
    def test_remove_field_name_str(self):
        region = Region.objects.create(pk=1, code='REGION', island_group=Region.ISLAND_GROUP_LUZON)
        self.assertTrue(str(region))

    def test_remove_field_code_repr(self):
        region = Region.objects.create(code='REGION', island_group=Region.ISLAND_GROUP_LUZON)
        province = Province.objects.create(name='PROVINCE', region=region)
        self.assertTrue(repr(province))

    def test_remove_field_code_name_repr(self):
        region = Region.objects.create(code='REGION', island_group=Region.ISLAND_GROUP_LUZON)
        province = Province.objects.create(name='PROVINCE', region=region)
        municipality = Municipality.objects.create(pk=1, province=province, is_city=False, is_capital=False)
        self.assertTrue(repr(municipality))

    #
    #   Remove field
    #
    def test_remove_field_specific_model_result(self):
        with self.assertRaises(FieldDoesNotExist):
            self.barangay_model._meta.get_field(self.REMOVED_FIELD_SPECIFIC)

    def test_remove_field_all_models_result(self):
        removed = []
        try:
            self.barangay_model._meta.get_field(self.REMOVED_FIELD_ALL)
            removed.append(False)
        except FieldDoesNotExist:
            removed.append(True)
        try:
            self.municipality_model._meta.get_field(self.REMOVED_FIELD_ALL)
            removed.append(False)
        except FieldDoesNotExist:
            removed.append(True)
        try:
            self.province_model._meta.get_field(self.REMOVED_FIELD_ALL)
            removed.append(False)
        except FieldDoesNotExist:
            removed.append(True)
        try:
            self.region_model._meta.get_field(self.REMOVED_FIELD_ALL)
            removed.append(False)
        except FieldDoesNotExist:
            removed.append(True)
        self.assertTrue(all(removed))

    def test_remove_field_specific_model_params_not_str(self):
        with self.assertRaises(TypeError):
            Barangay.remove_field('code', 123)

    def test_remove_field_all_models_params_not_str(self):
        with self.assertRaises(TypeError):
            PhilippineGeography.remove_field('code', 123)

    def test_remove_field_specific_model_not_found(self):
        with self.assertRaises(AttributeError):
            Barangay.remove_field('_')

    def test_remove_field_all_models_not_found(self):
        with self.assertRaises(AttributeError):
            PhilippineGeography.remove_field('_')

    #
    #   Add field
    #
    def test_add_field_specific_model_result(self):
        self.assertTrue(self.barangay_model._meta.get_field(self.ADDED_FIELD_SPECIFIC))

    def test_add_field_all_models_result(self):
        added = []
        try:
            self.barangay_model._meta.get_field(self.ADDED_FIELD_ALL)
            added.append(True)
        except FieldDoesNotExist:
            added.append(False)
        try:
            self.municipality_model._meta.get_field(self.ADDED_FIELD_ALL)
            added.append(True)
        except FieldDoesNotExist:
            added.append(False)
        try:
            self.province_model._meta.get_field(self.ADDED_FIELD_ALL)
            added.append(True)
        except FieldDoesNotExist:
            added.append(False)
        try:
            self.region_model._meta.get_field(self.ADDED_FIELD_ALL)
            added.append(True)
        except FieldDoesNotExist:
            added.append(False)
        self.assertTrue(all(added))



import os

BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))

SECRET_KEY = '_'  # nosec

INSTALLED_APPS = [
    'ph_geography',

    'tests',
]

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': os.path.join(BASE_DIR, 'db.sqlite3'),
    },
}



from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Region',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('code', models.CharField(max_length=10, unique=True, verbose_name='Code')),
                ('name', models.CharField(max_length=100, verbose_name='Name')),
                ('population', models.PositiveIntegerField(null=True, verbose_name='Population')),
                ('is_active', models.BooleanField(default=True, verbose_name='Is Active')),
                ('island_group', models.CharField(choices=[('L', 'LUZON'), ('V', 'VISAYAS'), ('M', 'MINDANAO')], max_length=1, verbose_name='Island Group')),
            ],
            options={
                'verbose_name': 'Region',
                'verbose_name_plural': 'Regions',
                'db_table': 'ph_geography_region',
            },
        ),
        migrations.CreateModel(
            name='Province',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('code', models.CharField(max_length=10, unique=True, verbose_name='Code')),
                ('name', models.CharField(max_length=100, verbose_name='Name')),
                ('population', models.PositiveIntegerField(null=True, verbose_name='Population')),
                ('is_active', models.BooleanField(default=True, verbose_name='Is Active')),
                ('income_class', models.CharField(blank=True, choices=[('1', '1ST'), ('2', '2ND'), ('3', '3RD'), ('4', '4TH'), ('5', '5TH'), ('6', '6TH'), ('S', 'SPECIAL')], max_length=1, verbose_name='Income Class')),
                ('region', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='provinces', related_query_name='province', to='ph_geography.region', verbose_name='Region')),
            ],
            options={
                'verbose_name': 'Province',
                'verbose_name_plural': 'Provinces',
                'db_table': 'ph_geography_province',
            },
        ),
        migrations.CreateModel(
            name='Municipality',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('code', models.CharField(max_length=10, unique=True, verbose_name='Code')),
                ('name', models.CharField(max_length=100, verbose_name='Name')),
                ('population', models.PositiveIntegerField(null=True, verbose_name='Population')),
                ('is_active', models.BooleanField(default=True, verbose_name='Is Active')),
                ('is_city', models.BooleanField(verbose_name='Is City')),
                ('is_capital', models.BooleanField(verbose_name='Is Capital')),
                ('city_class', models.CharField(blank=True, choices=[('C', 'CC'), ('I', 'ICC'), ('H', 'HUC')], max_length=1, verbose_name='City Class')),
                ('income_class', models.CharField(blank=True, choices=[('1', '1ST'), ('2', '2ND'), ('3', '3RD'), ('4', '4TH'), ('5', '5TH'), ('6', '6TH'), ('S', 'SPECIAL')], max_length=1, verbose_name='Income Class')),
                ('province', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='municipalities', related_query_name='municipality', to='ph_geography.province', verbose_name='Province')),
            ],
            options={
                'verbose_name': 'Municipality',
                'verbose_name_plural': 'Municipalities',
                'db_table': 'ph_geography_municipality',
            },
        ),
        migrations.CreateModel(
            name='Barangay',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('code', models.CharField(max_length=10, unique=True, verbose_name='Code')),
                ('name', models.CharField(max_length=100, verbose_name='Name')),
                ('population', models.PositiveIntegerField(null=True, verbose_name='Population')),
                ('is_active', models.BooleanField(default=True, verbose_name='Is Active')),
                ('is_urban', models.NullBooleanField(null=True, verbose_name='Is Urban')),
                ('municipality', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='barangays', related_query_name='barangays', to='ph_geography.municipality', verbose_name='Barangay')),
            ],
            options={
                'verbose_name': 'Barangay',
                'verbose_name_plural': 'Barangays',
                'db_table': 'ph_geography_barangay',
            },
        ),
    ]



import io

from setuptools import find_packages
from setuptools import setup

from ph_geography import __version__

with io.open('README.rst', 'rt', encoding='utf8') as f:
    README = f.read()

install_requires = ['django>=1.11', ]
tests_require = ['django-migration-testcase==0.0.15', ]
extras_require = {'test': tests_require, }

setup(
    name='django-ph-geography',
    version=__version__,
    description='Philippine Geography models for Django',
    long_description=README,
    long_description_content_type='text/x-rst',
    url='https://github.com/josemarimanio/django-ph-geography/',
    project_urls={
        'Source Code': 'https://github.com/josemarimanio/django-ph-geography/',
    },
    author='Jose Mari Manio',
    author_email='josemari.manio@tutamail.com',
    license='MIT',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Django :: 1.11',
        'Framework :: Django :: 2.2',
        'Framework :: Django :: 3.0',
        'Framework :: Django :: 3.1',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
    packages=find_packages(),
    include_package_data=True,
    python_requires='>=3.4',
    install_requires=install_requires,
    tests_require=tests_require,
    extras_require=extras_require,
)



from django.core import management
from django.core.management.base import BaseCommand


class Command(BaseCommand):
    help = 'Load all PH Geography fixtures.'

    def __init__(self):
        super(Command, self).__init__()
        self.app_name = 'ph_geography'
        self.region_fixtures = 'regions.json'
        self.province_fixtures = 'provinces.json'
        self.municipality_fixtures = 'municipalities.json'
        self.barangay_fixtures = 'barangays.json'

    def handle(self, *args, **kwargs):
        management.call_command('loaddata', self.region_fixtures, app=self.app_name)
        management.call_command('loaddata', self.province_fixtures, app=self.app_name)
        management.call_command('loaddata', self.municipality_fixtures, app=self.app_name)
        management.call_command('loaddata', self.barangay_fixtures, app=self.app_name)



from django.test import TestCase

from ph_geography.models import Barangay
from ph_geography.models import Municipality
from ph_geography.models import Province
from ph_geography.models import Region


class ModelTestCase(TestCase):
    """
    Test cases for django-ph-geography models

    Testing theses cases:
        * __str__() and __repr() implementation
        * Model field list
        * CharField choices
        * ForeignKey related_name
        * Model properties
    """
    fixtures = ('geography.json',)

    BARANGAY_NAME = 'DOÑA IMELDA'
    MUNICIPALITY_NAME = 'QUEZON CITY'
    PROVINCE_NAME = 'METRO MANILA'
    REGION_NAME = 'NATIONAL CAPITAL REGION (NCR)'

    @staticmethod
    def get_sorted_field_names(model):
        """Returns a sorted list of field names"""
        return sorted([field.name for field in model._meta.get_fields()])

    @staticmethod
    def repr_output(item):
        """Returns a string of simulated model __repr__ response"""
        return '<Code: {code}, {model}: {name}>'.format(
            code=getattr(item, 'code'),
            model=item.__class__.__name__,
            name=getattr(item, 'name'),
        )

    def setUp(self):
        self.barangay = Barangay.objects.get(name=self.BARANGAY_NAME)
        self.municipality = Municipality.objects.get(name=self.MUNICIPALITY_NAME)
        self.province = Province.objects.get(name=self.PROVINCE_NAME)
        self.region = Region.objects.get(name=self.REGION_NAME)

    #
    #   Region
    #
    def test_region_str(self):
        self.assertEqual(str(self.region), self.REGION_NAME)

    def test_region_repr(self):
        self.assertEqual(
            repr(self.region),
            self.repr_output(self.region)
        )

    def test_region_fields_list(self):
        self.assertEqual(
            self.get_sorted_field_names(Region),
            ['code', 'id', 'is_active', 'island_group', 'name', 'population', 'province']
        )

    def test_region_island_group_choices(self):
        self.assertEqual(
            Region.ISLAND_GROUP_CHOICES,
            (('L', 'LUZON'), ('V', 'VISAYAS'), ('M', 'MINDANAO'))
        )

    def test_region_provinces(self):
        self.assertTrue(self.region.provinces.all().count())

    #
    #   Province
    #
    def test_province_str(self):
        self.assertEqual(str(self.province), self.PROVINCE_NAME)

    def test_province_repr(self):
        self.assertEqual(repr(self.province), self.repr_output(self.province))

    def test_province_fields_list(self):
        self.assertEqual(
            self.get_sorted_field_names(Province),
            ['code', 'id', 'income_class', 'is_active', 'municipality', 'name', 'population', 'region', ]
        )

    def test_province_income_class_choices(self):
        self.assertEqual(
            Province.INCOME_CLASS_CHOICES,
            (('1', '1ST'), ('2', '2ND'), ('3', '3RD'), ('4', '4TH'), ('5', '5TH'), ('6', '6TH'), ('S', 'SPECIAL'))
        )

    def test_province_municipalities(self):
        self.assertTrue(self.province.municipalities.all().count())

    def test_province_island_group(self):
        self.assertEqual(self.province.island_group, self.region.island_group)

    #
    #   Municipality
    #
    def test_municipality_str(self):
        self.assertEqual(str(self.municipality), self.MUNICIPALITY_NAME)

    def test_municipality_repr(self):
        self.assertEqual(repr(self.municipality), self.repr_output(self.municipality))

    def test_municipality_fields_list(self):
        self.assertEqual(
            self.get_sorted_field_names(Municipality),
            ['barangays', 'city_class', 'code', 'id', 'income_class', 'is_active',
             'is_capital', 'is_city', 'name', 'population', 'province', ]
        )

    def test_municipality_city_class_choices(self):
        self.assertEqual(
            Municipality.CITY_CLASS_CHOICES,
            (('C', 'CC'), ('I', 'ICC'), ('H', 'HUC'))
        )

    def test_municipality_income_class_choices(self):
        self.assertEqual(
            Municipality.INCOME_CLASS_CHOICES,
            (('1', '1ST'), ('2', '2ND'), ('3', '3RD'), ('4', '4TH'), ('5', '5TH'), ('6', '6TH'), ('S', 'SPECIAL'))
        )

    def test_municipality_barangays(self):
        self.assertTrue(self.municipality.barangays.all().count())

    def test_municipality_region(self):
        self.assertEqual(self.municipality.region, self.region)

    def test_municipality_island_group(self):
        self.assertEqual(self.municipality.island_group, Region.ISLAND_GROUP_LUZON)

    #
    #   Barangay
    #
    def test_barangay_str(self):
        self.assertEqual(str(self.barangay), self.BARANGAY_NAME)

    def test_barangay_repr(self):
        self.assertEqual(repr(self.barangay), self.repr_output(self.barangay))

    def test_barangay_fields_list(self):
        self.assertEqual(
            self.get_sorted_field_names(Barangay),
            ['code', 'id', 'is_active', 'is_urban', 'municipality', 'name', 'population']
        )

    def test_barangay_province(self):
        self.assertEqual(self.barangay.province, self.province)

    def test_barangay_region(self):
        self.assertEqual(self.barangay.region, self.region)

    def test_barangay_island_group(self):
        self.assertEqual(self.barangay.island_group, self.region.island_group)
